@@ -17,27 +17,28 @@ package docker
 import (
 	"flag"
 	"fmt"
-	"path"
 	"regexp"
 	"strconv"
 	"strings"
 	"sync"
+	"time"
+
+	"github.com/blang/semver/v4"
+	dockertypes "github.com/docker/docker/api/types"
 
-	"github.com/blang/semver"
-	dockertypes "github.com/docker/engine-api/types"
 	"github.com/google/cadvisor/container"
+	dockerutil "github.com/google/cadvisor/container/docker/utils"
 	"github.com/google/cadvisor/container/libcontainer"
 	"github.com/google/cadvisor/devicemapper"
 	"github.com/google/cadvisor/fs"
 	info "github.com/google/cadvisor/info/v1"
 	"github.com/google/cadvisor/machine"
-	"github.com/google/cadvisor/manager/watcher"
-	dockerutil "github.com/google/cadvisor/utils/docker"
+	"github.com/google/cadvisor/watcher"
 	"github.com/google/cadvisor/zfs"
 
-	docker "github.com/docker/engine-api/client"
-	"github.com/golang/glog"
+	docker "github.com/docker/docker/client"
 	"golang.org/x/net/context"
+	"k8s.io/klog/v2"
 )
 
 var ArgDockerEndpoint = flag.String("docker", "unix:///var/run/docker.sock", "docker endpoint")
@@ -46,14 +47,16 @@ var ArgDockerCert = flag.String("docker-tls-cert", "cert.pem", "path to client c
 var ArgDockerKey = flag.String("docker-tls-key", "key.pem", "path to private key")
 var ArgDockerCA = flag.String("docker-tls-ca", "ca.pem", "path to trusted CA")
 
+var dockerEnvMetadataWhiteList = flag.String("docker_env_metadata_whitelist", "", "DEPRECATED: this flag will be removed, please use `env_metadata_whitelist`. A comma-separated list of environment variable keys matched with specified prefix that needs to be collected for docker containers")
+
 // The namespace under which Docker aliases are unique.
 const DockerNamespace = "docker"
 
-// Regexp that identifies docker cgroups, containers started with
-// --cgroup-parent have another prefix than 'docker'
-var dockerCgroupRegexp = regexp.MustCompile(`([a-z0-9]{64})`)
+// The retry times for getting docker root dir
+const rootDirRetries = 5
 
-var dockerEnvWhitelist = flag.String("docker_env_metadata_whitelist", "", "a comma-separated list of environment variable keys that needs to be collected for docker containers")
+// The retry period for getting docker root dir, Millisecond
+const rootDirRetryPeriod time.Duration = 1000 * time.Millisecond
 
 var (
 	// Basepath to all container specific information that libcontainer stores.
@@ -72,36 +75,43 @@ var (
 
 func RootDir() string {
 	dockerRootDirOnce.Do(func() {
-		status, err := Status()
-		if err == nil && status.RootDir != "" {
-			dockerRootDir = status.RootDir
-		} else {
+		for i := 0; i < rootDirRetries; i++ {
+			status, err := Status()
+			if err == nil && status.RootDir != "" {
+				dockerRootDir = status.RootDir
+				break
+			} else {
+				time.Sleep(rootDirRetryPeriod)
+			}
+		}
+		if dockerRootDir == "" {
 			dockerRootDir = *dockerRootDirFlag
 		}
 	})
 	return dockerRootDir
 }
 
-type storageDriver string
+type StorageDriver string
 
 const (
-	devicemapperStorageDriver storageDriver = "devicemapper"
-	aufsStorageDriver         storageDriver = "aufs"
-	overlayStorageDriver      storageDriver = "overlay"
-	overlay2StorageDriver     storageDriver = "overlay2"
-	zfsStorageDriver          storageDriver = "zfs"
+	DevicemapperStorageDriver StorageDriver = "devicemapper"
+	AufsStorageDriver         StorageDriver = "aufs"
+	OverlayStorageDriver      StorageDriver = "overlay"
+	Overlay2StorageDriver     StorageDriver = "overlay2"
+	ZfsStorageDriver          StorageDriver = "zfs"
+	VfsStorageDriver          StorageDriver = "vfs"
 )
 
 type dockerFactory struct {
 	machineInfoFactory info.MachineInfoFactory
 
-	storageDriver storageDriver
+	storageDriver StorageDriver
 	storageDir    string
 
 	client *docker.Client
 
 	// Information about the mounted cgroup subsystems.
-	cgroupSubsystems libcontainer.CgroupSubsystems
+	cgroupSubsystems map[string]string
 
 	// Information about mounted filesystems.
 	fsInfo fs.FsInfo
@@ -110,7 +120,7 @@ type dockerFactory struct {
 
 	dockerAPIVersion []int
 
-	ignoreMetrics container.MetricSet
+	includedMetrics container.MetricSet
 
 	thinPoolName    string
 	thinPoolWatcher *devicemapper.ThinPoolWatcher
@@ -118,70 +128,54 @@ type dockerFactory struct {
 	zfsWatcher *zfs.ZfsWatcher
 }
 
-func (self *dockerFactory) String() string {
+func (f *dockerFactory) String() string {
 	return DockerNamespace
 }
 
-func (self *dockerFactory) NewContainerHandler(name string, inHostNamespace bool) (handler container.ContainerHandler, err error) {
+func (f *dockerFactory) NewContainerHandler(name string, metadataEnvAllowList []string, inHostNamespace bool) (handler container.ContainerHandler, err error) {
 	client, err := Client()
 	if err != nil {
 		return
 	}
 
-	metadataEnvs := strings.Split(*dockerEnvWhitelist, ",")
+	dockerMetadataEnvAllowList := strings.Split(*dockerEnvMetadataWhiteList, ",")
+
+	// prefer using the unified metadataEnvAllowList
+	if len(metadataEnvAllowList) != 0 {
+		dockerMetadataEnvAllowList = metadataEnvAllowList
+	}
 
 	handler, err = newDockerContainerHandler(
 		client,
 		name,
-		self.machineInfoFactory,
-		self.fsInfo,
-		self.storageDriver,
-		self.storageDir,
-		&self.cgroupSubsystems,
+		f.machineInfoFactory,
+		f.fsInfo,
+		f.storageDriver,
+		f.storageDir,
+		f.cgroupSubsystems,
 		inHostNamespace,
-		metadataEnvs,
-		self.dockerVersion,
-		self.ignoreMetrics,
-		self.thinPoolName,
-		self.thinPoolWatcher,
-		self.zfsWatcher,
+		dockerMetadataEnvAllowList,
+		f.dockerVersion,
+		f.includedMetrics,
+		f.thinPoolName,
+		f.thinPoolWatcher,
+		f.zfsWatcher,
 	)
 	return
 }
 
-// Returns the Docker ID from the full container name.
-func ContainerNameToDockerId(name string) string {
-	id := path.Base(name)
-
-	if matches := dockerCgroupRegexp.FindStringSubmatch(id); matches != nil {
-		return matches[1]
-	}
-
-	return id
-}
-
-// isContainerName returns true if the cgroup with associated name
-// corresponds to a docker container.
-func isContainerName(name string) bool {
-	// always ignore .mount cgroup even if associated with docker and delegate to systemd
-	if strings.HasSuffix(name, ".mount") {
-		return false
-	}
-	return dockerCgroupRegexp.MatchString(path.Base(name))
-}
-
 // Docker handles all containers under /docker
-func (self *dockerFactory) CanHandleAndAccept(name string) (bool, bool, error) {
+func (f *dockerFactory) CanHandleAndAccept(name string) (bool, bool, error) {
 	// if the container is not associated with docker, we can't handle it or accept it.
-	if !isContainerName(name) {
+	if !dockerutil.IsContainerName(name) {
 		return false, false, nil
 	}
 
 	// Check if the container is known to docker and it is active.
-	id := ContainerNameToDockerId(name)
+	id := dockerutil.ContainerNameToId(name)
 
 	// We assume that if Inspect fails then the container is not known to docker.
-	ctnr, err := self.client.ContainerInspect(context.Background(), id)
+	ctnr, err := f.client.ContainerInspect(context.Background(), id)
 	if err != nil || !ctnr.State.Running {
 		return false, true, fmt.Errorf("error inspecting container: %v", err)
 	}
@@ -189,18 +183,18 @@ func (self *dockerFactory) CanHandleAndAccept(name string) (bool, bool, error) {
 	return true, true, nil
 }
 
-func (self *dockerFactory) DebugInfo() map[string][]string {
+func (f *dockerFactory) DebugInfo() map[string][]string {
 	return map[string][]string{}
 }
 
 var (
-	version_regexp_string    = `(\d+)\.(\d+)\.(\d+)`
-	version_re               = regexp.MustCompile(version_regexp_string)
-	apiversion_regexp_string = `(\d+)\.(\d+)`
-	apiversion_re            = regexp.MustCompile(apiversion_regexp_string)
+	versionRegexpString    = `(\d+)\.(\d+)\.(\d+)`
+	VersionRe              = regexp.MustCompile(versionRegexpString)
+	apiVersionRegexpString = `(\d+)\.(\d+)`
+	apiVersionRe           = regexp.MustCompile(apiVersionRegexpString)
 )
 
-func startThinPoolWatcher(dockerInfo *dockertypes.Info) (*devicemapper.ThinPoolWatcher, error) {
+func StartThinPoolWatcher(dockerInfo *dockertypes.Info) (*devicemapper.ThinPoolWatcher, error) {
 	_, err := devicemapper.ThinLsBinaryPresent()
 	if err != nil {
 		return nil, err
@@ -233,7 +227,7 @@ func startThinPoolWatcher(dockerInfo *dockertypes.Info) (*devicemapper.ThinPoolW
 	return thinPoolWatcher, nil
 }
 
-func startZfsWatcher(dockerInfo *dockertypes.Info) (*zfs.ZfsWatcher, error) {
+func StartZfsWatcher(dockerInfo *dockertypes.Info) (*zfs.ZfsWatcher, error) {
 	filesystem, err := dockerutil.DockerZfsFilesystem(*dockerInfo)
 	if err != nil {
 		return nil, err
@@ -255,7 +249,7 @@ func ensureThinLsKernelVersion(kernelVersion string) error {
 	// thin_ls to work without corrupting the thin pool
 	minRhel7KernelVersion := semver.MustParse("3.10.0")
 
-	matches := version_re.FindStringSubmatch(kernelVersion)
+	matches := VersionRe.FindStringSubmatch(kernelVersion)
 	if len(matches) < 4 {
 		return fmt.Errorf("error parsing kernel version: %q is not a semver", kernelVersion)
 	}
@@ -309,23 +303,23 @@ func ensureThinLsKernelVersion(kernelVersion string) error {
 }
 
 // Register root container before running this function!
-func Register(factory info.MachineInfoFactory, fsInfo fs.FsInfo, ignoreMetrics container.MetricSet) error {
+func Register(factory info.MachineInfoFactory, fsInfo fs.FsInfo, includedMetrics container.MetricSet) error {
 	client, err := Client()
 	if err != nil {
 		return fmt.Errorf("unable to communicate with docker daemon: %v", err)
 	}
 
-	dockerInfo, err := ValidateInfo()
+	dockerInfo, err := ValidateInfo(Info, VersionString)
 	if err != nil {
 		return fmt.Errorf("failed to validate Docker info: %v", err)
 	}
 
 	// Version already validated above, assume no error here.
-	dockerVersion, _ := parseVersion(dockerInfo.ServerVersion, version_re, 3)
+	dockerVersion, _ := ParseVersion(dockerInfo.ServerVersion, VersionRe, 3)
 
 	dockerAPIVersion, _ := APIVersion()
 
-	cgroupSubsystems, err := libcontainer.GetCgroupSubsystems()
+	cgroupSubsystems, err := libcontainer.GetCgroupSubsystems(includedMetrics)
 	if err != nil {
 		return fmt.Errorf("failed to get cgroup subsystems: %v", err)
 	}
@@ -333,26 +327,29 @@ func Register(factory info.MachineInfoFactory, fsInfo fs.FsInfo, ignoreMetrics c
 	var (
 		thinPoolWatcher *devicemapper.ThinPoolWatcher
 		thinPoolName    string
+		zfsWatcher      *zfs.ZfsWatcher
 	)
-	if storageDriver(dockerInfo.Driver) == devicemapperStorageDriver {
-		thinPoolWatcher, err = startThinPoolWatcher(dockerInfo)
-		if err != nil {
-			glog.Errorf("devicemapper filesystem stats will not be reported: %v", err)
+	if includedMetrics.Has(container.DiskUsageMetrics) {
+		if StorageDriver(dockerInfo.Driver) == DevicemapperStorageDriver {
+			thinPoolWatcher, err = StartThinPoolWatcher(dockerInfo)
+			if err != nil {
+				klog.Errorf("devicemapper filesystem stats will not be reported: %v", err)
+			}
+
+			// Safe to ignore error - driver status should always be populated.
+			status, _ := StatusFromDockerInfo(*dockerInfo)
+			thinPoolName = status.DriverStatus[dockerutil.DriverStatusPoolName]
 		}
 
-		status := StatusFromDockerInfo(*dockerInfo)
-		thinPoolName = status.DriverStatus[dockerutil.DriverStatusPoolName]
-	}
-
-	var zfsWatcher *zfs.ZfsWatcher
-	if storageDriver(dockerInfo.Driver) == zfsStorageDriver {
-		zfsWatcher, err = startZfsWatcher(dockerInfo)
-		if err != nil {
-			glog.Errorf("zfs filesystem stats will not be reported: %v", err)
+		if StorageDriver(dockerInfo.Driver) == ZfsStorageDriver {
+			zfsWatcher, err = StartZfsWatcher(dockerInfo)
+			if err != nil {
+				klog.Errorf("zfs filesystem stats will not be reported: %v", err)
+			}
 		}
 	}
 
-	glog.Infof("Registering Docker factory")
+	klog.V(1).Infof("Registering Docker factory")
 	f := &dockerFactory{
 		cgroupSubsystems:   cgroupSubsystems,
 		client:             client,
@@ -360,9 +357,9 @@ func Register(factory info.MachineInfoFactory, fsInfo fs.FsInfo, ignoreMetrics c
 		dockerAPIVersion:   dockerAPIVersion,
 		fsInfo:             fsInfo,
 		machineInfoFactory: factory,
-		storageDriver:      storageDriver(dockerInfo.Driver),
+		storageDriver:      StorageDriver(dockerInfo.Driver),
 		storageDir:         RootDir(),
-		ignoreMetrics:      ignoreMetrics,
+		includedMetrics:    includedMetrics,
 		thinPoolName:       thinPoolName,
 		thinPoolWatcher:    thinPoolWatcher,
 		zfsWatcher:         zfsWatcher,
