@@ -16,21 +16,33 @@ package fs
 
 import (
 	"errors"
-	"io/ioutil"
 	"os"
 	"reflect"
 	"testing"
-	"time"
 
-	"github.com/docker/docker/pkg/mount"
+	mount "github.com/moby/sys/mountinfo"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
 
+func TestMountInfoFromDir(t *testing.T) {
+	as := assert.New(t)
+	fsInfo := &RealFsInfo{
+		mounts: map[string]mount.Info{
+			"/": {},
+		},
+	}
+	testDirs := []string{"/var/lib/kubelet", "/var/lib/rancher"}
+	for _, testDir := range testDirs {
+		_, found := fsInfo.mountInfoFromDir(testDir)
+		as.True(found, "failed to find MountInfo %s from FsInfo %s", testDir, fsInfo)
+	}
+}
+
 func TestGetDiskStatsMap(t *testing.T) {
 	diskStatsMap, err := getDiskStatsMap("test_resources/diskstats")
 	if err != nil {
-		t.Errorf("Error calling getDiskStatMap %s", err)
+		t.Errorf("Error calling getDiskStatsMap %s", err)
 	}
 	if len(diskStatsMap) != 30 {
 		t.Errorf("diskStatsMap %+v not valid", diskStatsMap)
@@ -79,6 +91,22 @@ func TestGetDiskStatsMap(t *testing.T) {
 	}
 }
 
+func TestGetDiskStatsMapMajorMinorNum(t *testing.T) {
+	diskStatsMap, err := getDiskStatsMap("test_resources/diskstats")
+	if err != nil {
+		t.Errorf("Error calling getDiskStatsMap %s", err)
+	}
+	if len(diskStatsMap) != 30 {
+		t.Errorf("diskStatsMap %+v not valid", diskStatsMap)
+	}
+
+	if stat, ok := diskStatsMap["/dev/dm-0"]; ok {
+		if stat.MajorNum != 252 && stat.MinorNum != 1 {
+			t.Fatalf("getDiskStatsMap did not return correct major (%d) and minor (%d) numbers", stat.MajorNum, stat.MinorNum)
+		}
+	}
+}
+
 func TestFileNotExist(t *testing.T) {
 	_, err := getDiskStatsMap("/file_does_not_exist")
 	if err != nil {
@@ -90,38 +118,37 @@ func TestDirDiskUsage(t *testing.T) {
 	as := assert.New(t)
 	fsInfo, err := NewFsInfo(Context{})
 	as.NoError(err)
-	dir, err := ioutil.TempDir(os.TempDir(), "")
+	dir := t.TempDir()
 	as.NoError(err)
 	defer os.RemoveAll(dir)
 	dataSize := 1024 * 100 //100 KB
 	b := make([]byte, dataSize)
-	f, err := ioutil.TempFile(dir, "")
+	f, err := os.CreateTemp(dir, "")
 	as.NoError(err)
-	as.NoError(ioutil.WriteFile(f.Name(), b, 0700))
+	as.NoError(os.WriteFile(f.Name(), b, 0700))
 	fi, err := f.Stat()
 	as.NoError(err)
 	expectedSize := uint64(fi.Size())
-	size, err := fsInfo.GetDirDiskUsage(dir, time.Minute)
+	usage, err := fsInfo.GetDirUsage(dir)
 	as.NoError(err)
-	as.True(expectedSize <= size, "expected dir size to be at-least %d; got size: %d", expectedSize, size)
+	as.True(expectedSize <= usage.Bytes, "expected dir size to be at-least %d; got size: %d", expectedSize, usage.Bytes)
 }
 
 func TestDirInodeUsage(t *testing.T) {
 	as := assert.New(t)
 	fsInfo, err := NewFsInfo(Context{})
 	as.NoError(err)
-	dir, err := ioutil.TempDir(os.TempDir(), "")
-	as.NoError(err)
+	dir := t.TempDir()
 	defer os.RemoveAll(dir)
 	numFiles := 1000
 	for i := 0; i < numFiles; i++ {
-		_, err := ioutil.TempFile(dir, "")
+		_, err := os.MkdirTemp(dir, "")
 		require.NoError(t, err)
 	}
-	inodes, err := fsInfo.GetDirInodeUsage(dir, time.Minute)
+	usage, err := fsInfo.GetDirUsage(dir)
 	as.NoError(err)
 	// We sould get numFiles+1 inodes, since we get 1 inode for each file, plus 1 for the directory
-	as.True(uint64(numFiles+1) == inodes, "expected inodes in dir to be %d; got inodes: %d", numFiles+1, inodes)
+	as.True(uint64(numFiles+1) == usage.Inodes, "expected inodes in dir to be %d; got inodes: %d", numFiles+1, usage.Inodes)
 }
 
 var dmStatusTests = []struct {
@@ -344,6 +371,23 @@ func TestAddDockerImagesLabel(t *testing.T) {
 		expectedDockerDevice           string
 		expectedPartition              *partition
 	}{
+		{
+			name:   "single partition, no dedicated image fs",
+			driver: "overlay2",
+			mounts: []*mount.Info{
+				{
+					Source:     "/dev/root",
+					Mountpoint: "/",
+					FSType:     "ext4",
+				},
+				{
+					Source:     "/sys/fs/cgroup",
+					Mountpoint: "/sys/fs/cgroup",
+					FSType:     "tmpfs",
+				},
+			},
+			expectedDockerDevice: "/dev/root",
+		},
 		{
 			name:         "devicemapper, not loopback",
 			driver:       "devicemapper",
@@ -353,7 +397,7 @@ func TestAddDockerImagesLabel(t *testing.T) {
 				{
 					Source:     "/dev/mapper/vg_vagrant-lv_root",
 					Mountpoint: "/",
-					Fstype:     "devicemapper",
+					FSType:     "devicemapper",
 				},
 			},
 			expectedDockerDevice: "vg_vagrant-docker--pool",
@@ -372,7 +416,7 @@ func TestAddDockerImagesLabel(t *testing.T) {
 				{
 					Source:     "/dev/mapper/vg_vagrant-lv_root",
 					Mountpoint: "/",
-					Fstype:     "devicemapper",
+					FSType:     "devicemapper",
 				},
 				{
 					Source:     "/dev/sdb1",
@@ -382,59 +426,62 @@ func TestAddDockerImagesLabel(t *testing.T) {
 			expectedDockerDevice: "/dev/sdb1",
 		},
 		{
-			name: "multiple mounts - innermost check",
+			name:   "multiple mounts - innermost check",
+			driver: "overlay2",
 			mounts: []*mount.Info{
 				{
 					Source:     "/dev/sda1",
 					Mountpoint: "/",
-					Fstype:     "ext4",
+					FSType:     "ext4",
 				},
 				{
 					Source:     "/dev/sdb1",
 					Mountpoint: "/var/lib/docker",
-					Fstype:     "ext4",
+					FSType:     "ext4",
 				},
 				{
 					Source:     "/dev/sdb2",
 					Mountpoint: "/var/lib/docker/btrfs",
-					Fstype:     "btrfs",
+					FSType:     "btrfs",
 				},
 			},
 			expectedDockerDevice: "/dev/sdb2",
 		},
 		{
-			name: "root fs inside container, docker-images bindmount",
+			name:   "root fs inside container, docker-images bindmount",
+			driver: "overlay2",
 			mounts: []*mount.Info{
 				{
 					Source:     "overlay",
 					Mountpoint: "/",
-					Fstype:     "overlay",
+					FSType:     "overlay",
 				},
 				{
 					Source:     "/dev/sda1",
 					Mountpoint: "/var/lib/docker",
-					Fstype:     "ext4",
+					FSType:     "ext4",
 				},
 			},
 			expectedDockerDevice: "/dev/sda1",
 		},
 		{
-			name: "[overlay2] root fs inside container - /var/lib/docker bindmount",
+			name:   "[overlay2] root fs inside container - /var/lib/docker bindmount",
+			driver: "overlay2",
 			mounts: []*mount.Info{
 				{
 					Source:     "overlay",
 					Mountpoint: "/",
-					Fstype:     "overlay",
+					FSType:     "overlay",
 				},
 				{
 					Source:     "/dev/sdb1",
 					Mountpoint: "/var/lib/docker",
-					Fstype:     "ext4",
+					FSType:     "ext4",
 				},
 				{
 					Source:     "/dev/sdb2",
 					Mountpoint: "/var/lib/docker/overlay2",
-					Fstype:     "ext4",
+					FSType:     "ext4",
 				},
 			},
 			expectedDockerDevice: "/dev/sdb2",
@@ -473,6 +520,116 @@ func TestAddDockerImagesLabel(t *testing.T) {
 	}
 }
 
+func TestAddCrioImagesLabel(t *testing.T) {
+	tests := []struct {
+		name                   string
+		driver                 string
+		driverStatus           map[string]string
+		dmsetupTable           string
+		mounts                 []*mount.Info
+		imageStore             string
+		expectedCrioImages     string
+		expectedCrioContainers string
+		expectedPartition      *partition
+	}{
+		{
+			name:   "single partition, no dedicated image fs",
+			driver: "overlay2",
+			mounts: []*mount.Info{
+				{
+					Source:     "/dev/root",
+					Mountpoint: "/",
+					FSType:     "ext4",
+				},
+				{
+					Source:     "/sys/fs/cgroup",
+					Mountpoint: "/sys/fs/cgroup",
+					FSType:     "tmpfs",
+				},
+			},
+			expectedCrioImages:     "/dev/root",
+			expectedCrioContainers: "",
+		},
+		{
+			name:   "root fs inside container, docker-images bindmount",
+			driver: "overlay2",
+			mounts: []*mount.Info{
+				{
+					Source:     "overlay",
+					Mountpoint: "/",
+					FSType:     "overlay",
+				},
+				{
+					Source:     "/dev/sda1",
+					Mountpoint: "/var/lib/container",
+					FSType:     "ext4",
+				},
+			},
+			expectedCrioImages: "/dev/sda1",
+		},
+		{
+			name:   "[overlay2] image and container separate",
+			driver: "overlay2",
+			mounts: []*mount.Info{
+				{
+					Source:     "/dev/sdb1",
+					Mountpoint: "/imagestore",
+					FSType:     "ext4",
+				},
+				{
+					Source:     "/dev/sdb2",
+					Mountpoint: "/var/lib/container",
+					FSType:     "ext4",
+				},
+			},
+			expectedCrioImages:     "/dev/sdb1",
+			expectedCrioContainers: "/dev/sdb2",
+			imageStore:             "/imagestore",
+		},
+	}
+
+	for _, tt := range tests {
+		fsInfo := &RealFsInfo{
+			labels:     map[string]string{},
+			partitions: map[string]partition{},
+			dmsetup: &testDmsetup{
+				data: []byte(tt.dmsetupTable),
+			},
+		}
+
+		context := Context{
+			Crio: CrioContext{
+				Root:       "/var/lib/container",
+				ImageStore: tt.imageStore,
+				Driver:     "overlay",
+			},
+		}
+
+		fsInfo.addCrioImagesLabel(context, tt.mounts)
+
+		if tt.imageStore != "" {
+			if e, a := tt.expectedCrioImages, fsInfo.labels[LabelCrioImages]; e != a {
+				t.Errorf("%s: docker device: expected %q, got %q", tt.name, e, a)
+			}
+
+			if e, a := tt.expectedCrioContainers, fsInfo.labels[LabelCrioContainers]; e != a {
+				t.Errorf("%s: docker device: expected %q, got %q", tt.name, e, a)
+			}
+
+		}
+		if tt.imageStore == "" {
+			if e, a := tt.expectedCrioImages, fsInfo.labels[LabelCrioImages]; e != a {
+				t.Errorf("%s: docker device: expected %q, got %q", tt.name, e, a)
+			}
+
+		}
+
+		if tt.expectedPartition == nil {
+			continue
+		}
+	}
+}
+
 func TestProcessMounts(t *testing.T) {
 	tests := []struct {
 		name             string
@@ -483,16 +640,15 @@ func TestProcessMounts(t *testing.T) {
 		{
 			name: "unsupported fs types",
 			mounts: []*mount.Info{
-				{Fstype: "overlay"},
-				{Fstype: "somethingelse"},
+				{FSType: "somethingelse"},
 			},
 			expected: map[string]partition{},
 		},
 		{
 			name: "avoid bind mounts",
 			mounts: []*mount.Info{
-				{Root: "/", Mountpoint: "/", Source: "/dev/sda1", Fstype: "xfs", Major: 253, Minor: 0},
-				{Root: "/foo", Mountpoint: "/bar", Source: "/dev/sda1", Fstype: "xfs", Major: 253, Minor: 0},
+				{Root: "/", Mountpoint: "/", Source: "/dev/sda1", FSType: "xfs", Major: 253, Minor: 0},
+				{Root: "/foo", Mountpoint: "/bar", Source: "/dev/sda1", FSType: "xfs", Major: 253, Minor: 0},
 			},
 			expected: map[string]partition{
 				"/dev/sda1": {fsType: "xfs", mountpoint: "/", major: 253, minor: 0},
@@ -501,9 +657,9 @@ func TestProcessMounts(t *testing.T) {
 		{
 			name: "exclude prefixes",
 			mounts: []*mount.Info{
-				{Root: "/", Mountpoint: "/someother", Source: "/dev/sda1", Fstype: "xfs", Major: 253, Minor: 2},
-				{Root: "/", Mountpoint: "/", Source: "/dev/sda2", Fstype: "xfs", Major: 253, Minor: 0},
-				{Root: "/", Mountpoint: "/excludeme", Source: "/dev/sda3", Fstype: "xfs", Major: 253, Minor: 1},
+				{Root: "/", Mountpoint: "/someother", Source: "/dev/sda1", FSType: "xfs", Major: 253, Minor: 2},
+				{Root: "/", Mountpoint: "/", Source: "/dev/sda2", FSType: "xfs", Major: 253, Minor: 0},
+				{Root: "/", Mountpoint: "/excludeme", Source: "/dev/sda3", FSType: "xfs", Major: 253, Minor: 1},
 			},
 			excludedPrefixes: []string{"/exclude", "/some"},
 			expected: map[string]partition{
@@ -513,18 +669,26 @@ func TestProcessMounts(t *testing.T) {
 		{
 			name: "supported fs types",
 			mounts: []*mount.Info{
-				{Root: "/", Mountpoint: "/a", Source: "/dev/sda", Fstype: "ext3", Major: 253, Minor: 0},
-				{Root: "/", Mountpoint: "/b", Source: "/dev/sdb", Fstype: "ext4", Major: 253, Minor: 1},
-				{Root: "/", Mountpoint: "/c", Source: "/dev/sdc", Fstype: "btrfs", Major: 253, Minor: 2},
-				{Root: "/", Mountpoint: "/d", Source: "/dev/sdd", Fstype: "xfs", Major: 253, Minor: 3},
-				{Root: "/", Mountpoint: "/e", Source: "/dev/sde", Fstype: "zfs", Major: 253, Minor: 4},
+				{Root: "/", Mountpoint: "/a", Source: "/dev/sda", FSType: "ext3", Major: 253, Minor: 0},
+				{Root: "/", Mountpoint: "/b", Source: "/dev/sdb", FSType: "ext4", Major: 253, Minor: 1},
+				{Root: "/", Mountpoint: "/c", Source: "/dev/sdc", FSType: "btrfs", Major: 253, Minor: 2},
+				{Root: "/", Mountpoint: "/d", Source: "/dev/sdd", FSType: "xfs", Major: 253, Minor: 3},
+				{Root: "/", Mountpoint: "/e", Source: "/dev/sde", FSType: "zfs", Major: 253, Minor: 4},
+				{Root: "/", Mountpoint: "/f", Source: "overlay", FSType: "overlay", Major: 253, Minor: 5},
+				{Root: "/", Mountpoint: "/g", Source: "127.0.0.1:/nfs", FSType: "nfs4", Major: 253, Minor: 6},
+				{Root: "/", Mountpoint: "/test1", Source: "tmpfs", FSType: "tmpfs", Major: 253, Minor: 4},
+				{Root: "/", Mountpoint: "/test2", Source: "tmpfs", FSType: "tmpfs", Major: 253, Minor: 4},
 			},
 			expected: map[string]partition{
-				"/dev/sda": {fsType: "ext3", mountpoint: "/a", major: 253, minor: 0},
-				"/dev/sdb": {fsType: "ext4", mountpoint: "/b", major: 253, minor: 1},
-				"/dev/sdc": {fsType: "btrfs", mountpoint: "/c", major: 253, minor: 2},
-				"/dev/sdd": {fsType: "xfs", mountpoint: "/d", major: 253, minor: 3},
-				"/dev/sde": {fsType: "zfs", mountpoint: "/e", major: 253, minor: 4},
+				"/dev/sda":       {fsType: "ext3", mountpoint: "/a", major: 253, minor: 0},
+				"/dev/sdb":       {fsType: "ext4", mountpoint: "/b", major: 253, minor: 1},
+				"/dev/sdc":       {fsType: "btrfs", mountpoint: "/c", major: 253, minor: 2},
+				"/dev/sdd":       {fsType: "xfs", mountpoint: "/d", major: 253, minor: 3},
+				"/dev/sde":       {fsType: "zfs", mountpoint: "/e", major: 253, minor: 4},
+				"overlay_253-5":  {fsType: "overlay", mountpoint: "/f", major: 253, minor: 5},
+				"127.0.0.1:/nfs": {fsType: "nfs4", mountpoint: "/g", major: 253, minor: 6},
+				"/test1":         {fsType: "tmpfs", mountpoint: "/test1", major: 253, minor: 4},
+				"/test2":         {fsType: "tmpfs", mountpoint: "/test2", major: 253, minor: 4},
 			},
 		},
 	}
