@@ -22,9 +22,10 @@ import (
 	"time"
 
 	info "github.com/google/cadvisor/info/v1"
-	"github.com/google/cadvisor/info/v2"
+	v2 "github.com/google/cadvisor/info/v2"
 	"github.com/google/cadvisor/integration/framework"
 
+	"github.com/opencontainers/runc/libcontainer/cgroups"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -63,29 +64,23 @@ func waitForContainer(alias string, fm framework.Framework) {
 	require.NoError(fm.T(), err, "Timed out waiting for container %q to be available in cAdvisor: %v", alias, err)
 }
 
-func getDockerMinorVersion(fm framework.Framework) int {
-	val, err := strconv.Atoi(fm.Docker().Version()[1])
-	assert.Nil(fm.T(), err)
-	return val
-}
-
 // A Docker container in /docker/<ID>
 func TestDockerContainerById(t *testing.T) {
 	fm := framework.New(t)
 	defer fm.Cleanup()
 
-	containerId := fm.Docker().RunPause()
+	containerID := fm.Docker().RunPause()
 
 	// Wait for the container to show up.
-	waitForContainer(containerId, fm)
+	waitForContainer(containerID, fm)
 
 	request := &info.ContainerInfoRequest{
 		NumStats: 1,
 	}
-	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerId, request)
+	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerID, request)
 	require.NoError(t, err)
 
-	sanityCheck(containerId, containerInfo, t)
+	sanityCheck(containerID, containerInfo, t)
 }
 
 // A Docker container in /docker/<name>
@@ -95,7 +90,7 @@ func TestDockerContainerByName(t *testing.T) {
 
 	containerName := fmt.Sprintf("test-docker-container-by-name-%d", os.Getpid())
 	fm.Docker().Run(framework.DockerRunArgs{
-		Image: "kubernetes/pause",
+		Image: "registry.k8s.io/pause",
 		Args:  []string{"--name", containerName},
 	})
 
@@ -130,10 +125,10 @@ func TestGetAllDockerContainers(t *testing.T) {
 	defer fm.Cleanup()
 
 	// Wait for the containers to show up.
-	containerId1 := fm.Docker().RunPause()
-	containerId2 := fm.Docker().RunPause()
-	waitForContainer(containerId1, fm)
-	waitForContainer(containerId2, fm)
+	containerID1 := fm.Docker().RunPause()
+	containerID2 := fm.Docker().RunPause()
+	waitForContainer(containerID1, fm)
+	waitForContainer(containerID2, fm)
 
 	request := &info.ContainerInfoRequest{
 		NumStats: 1,
@@ -144,8 +139,8 @@ func TestGetAllDockerContainers(t *testing.T) {
 	if len(containersInfo) < 2 {
 		t.Fatalf("At least 2 Docker containers should exist, received %d: %+v", len(containersInfo), containersInfo)
 	}
-	sanityCheck(containerId1, findContainer(containerId1, containersInfo, t), t)
-	sanityCheck(containerId2, findContainer(containerId2, containersInfo, t), t)
+	sanityCheck(containerID1, findContainer(containerID1, containersInfo, t), t)
+	sanityCheck(containerID2, findContainer(containerID2, containersInfo, t), t)
 }
 
 // Check expected properties of a Docker container.
@@ -154,24 +149,24 @@ func TestBasicDockerContainer(t *testing.T) {
 	defer fm.Cleanup()
 
 	containerName := fmt.Sprintf("test-basic-docker-container-%d", os.Getpid())
-	containerId := fm.Docker().Run(framework.DockerRunArgs{
-		Image: "kubernetes/pause",
+	containerID := fm.Docker().Run(framework.DockerRunArgs{
+		Image: "registry.k8s.io/pause",
 		Args: []string{
 			"--name", containerName,
 		},
 	})
 
 	// Wait for the container to show up.
-	waitForContainer(containerId, fm)
+	waitForContainer(containerID, fm)
 
 	request := &info.ContainerInfoRequest{
 		NumStats: 1,
 	}
-	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerId, request)
+	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerID, request)
 	require.NoError(t, err)
 
 	// Check that the contianer is known by both its name and ID.
-	sanityCheck(containerId, containerInfo, t)
+	sanityCheck(containerID, containerInfo, t)
 	sanityCheck(containerName, containerInfo, t)
 
 	assert.Empty(t, containerInfo.Subcontainers, "Should not have subcontainers")
@@ -188,20 +183,16 @@ func TestDockerContainerSpec(t *testing.T) {
 		cpuShares   = uint64(2048)
 		cpuMask     = "0"
 		memoryLimit = uint64(1 << 30) // 1GB
-		image       = "kubernetes/pause"
+		image       = "registry.k8s.io/pause"
 		env         = map[string]string{"test_var": "FOO"}
 		labels      = map[string]string{"bar": "baz"}
 	)
 
-	cpusetArg := "--cpuset"
-	if getDockerMinorVersion(fm) >= 10 {
-		cpusetArg = "--cpuset-cpus"
-	}
-	containerId := fm.Docker().Run(framework.DockerRunArgs{
+	containerID := fm.Docker().Run(framework.DockerRunArgs{
 		Image: image,
 		Args: []string{
 			"--cpu-shares", strconv.FormatUint(cpuShares, 10),
-			cpusetArg, cpuMask,
+			"--cpuset-cpus", cpuMask,
 			"--memory", strconv.FormatUint(memoryLimit, 10),
 			"--env", "TEST_VAR=FOO",
 			"--label", "bar=baz",
@@ -209,19 +200,32 @@ func TestDockerContainerSpec(t *testing.T) {
 	})
 
 	// Wait for the container to show up.
-	waitForContainer(containerId, fm)
+	waitForContainer(containerID, fm)
 
 	request := &info.ContainerInfoRequest{
 		NumStats: 1,
 	}
-	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerId, request)
+	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerID, request)
 	require.NoError(t, err)
-	sanityCheck(containerId, containerInfo, t)
+	sanityCheck(containerID, containerInfo, t)
 
 	assert := assert.New(t)
 
 	assert.True(containerInfo.Spec.HasCpu, "CPU should be isolated")
-	assert.Equal(cpuShares, containerInfo.Spec.Cpu.Limit, "Container should have %d shares, has %d", cpuShares, containerInfo.Spec.Cpu.Limit)
+	if cgroups.IsCgroup2UnifiedMode() {
+		// cpu shares are rounded slighly on cgroupv2 due to conversion between cgroupv1 (cpu.shares) and cgroupv2 (cpu.weight)
+		// When container is created via docker, runc will convert cpu shares to cpu.weight https://github.com/opencontainers/runc/blob/d11f4d756e85ece5cdba8bb69f8bd4db3cdcbeab/libcontainer/cgroups/utils.go#L423-L428
+		// And cAdvisor will convert cpu.weight back to cpu shares in https://github.com/google/cadvisor/blob/24e7a9883d12f944fd4403861707f4bafcaf4f3d/container/common/helpers.go#L249-L260
+		// Worked example:
+		// cpuShares = 2048 (input to docker --cpu-shares)
+		// cpuWeight = int((1 + ((cpuShares-2)*9999)/262142))=79 (conversion done by runc)
+		// cpuWeight back to cpuShares = int(2 + ((cpuWeight-1)*262142)/9999)= 2046
+		var cgroupV2Shares uint64 = 2046
+		assert.Equal(cgroupV2Shares, containerInfo.Spec.Cpu.Limit, "Container should have %d shares, has %d", cgroupV2Shares, containerInfo.Spec.Cpu.Limit)
+	} else {
+		assert.Equal(cpuShares, containerInfo.Spec.Cpu.Limit, "Container should have %d shares, has %d", cpuShares, containerInfo.Spec.Cpu.Limit)
+	}
+
 	assert.Equal(cpuMask, containerInfo.Spec.Cpu.Mask, "Cpu mask should be %q, but is %q", cpuMask, containerInfo.Spec.Cpu.Mask)
 	assert.True(containerInfo.Spec.HasMemory, "Memory should be isolated")
 	assert.Equal(memoryLimit, containerInfo.Spec.Memory.Limit, "Container should have memory limit of %d, has %d", memoryLimit, containerInfo.Spec.Memory.Limit)
@@ -239,20 +243,20 @@ func TestDockerContainerCpuStats(t *testing.T) {
 	defer fm.Cleanup()
 
 	// Wait for the container to show up.
-	containerId := fm.Docker().RunBusybox("ping", "www.google.com")
-	waitForContainer(containerId, fm)
+	containerID := fm.Docker().RunBusybox("ping", "www.google.com")
+	waitForContainer(containerID, fm)
 
 	request := &info.ContainerInfoRequest{
 		NumStats: 1,
 	}
-	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerId, request)
+	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerID, request)
 	if err != nil {
 		t.Fatal(err)
 	}
-	sanityCheck(containerId, containerInfo, t)
+	sanityCheck(containerID, containerInfo, t)
 
 	// Checks for CpuStats.
-	checkCpuStats(t, containerInfo.Stats[0].Cpu)
+	checkCPUStats(t, containerInfo.Stats[0].Cpu)
 }
 
 // Check the memory ContainerStats.
@@ -261,15 +265,15 @@ func TestDockerContainerMemoryStats(t *testing.T) {
 	defer fm.Cleanup()
 
 	// Wait for the container to show up.
-	containerId := fm.Docker().RunBusybox("ping", "www.google.com")
-	waitForContainer(containerId, fm)
+	containerID := fm.Docker().RunBusybox("ping", "www.google.com")
+	waitForContainer(containerID, fm)
 
 	request := &info.ContainerInfoRequest{
 		NumStats: 1,
 	}
-	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerId, request)
+	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerID, request)
 	require.NoError(t, err)
-	sanityCheck(containerId, containerInfo, t)
+	sanityCheck(containerID, containerInfo, t)
 
 	// Checks for MemoryStats.
 	checkMemoryStats(t, containerInfo.Stats[0].Memory)
@@ -281,26 +285,38 @@ func TestDockerContainerNetworkStats(t *testing.T) {
 	defer fm.Cleanup()
 
 	// Wait for the container to show up.
-	containerId := fm.Docker().RunBusybox("watch", "-n1", "wget", "http://www.google.com/")
-	waitForContainer(containerId, fm)
+	containerID := fm.Docker().RunBusybox("watch", "-n1", "wget", "http://www.google.com/")
+	waitForContainer(containerID, fm)
 
-	time.Sleep(10 * time.Second)
+	// Wait for at least one additional housekeeping interval
+	time.Sleep(20 * time.Second)
 	request := &info.ContainerInfoRequest{
 		NumStats: 1,
 	}
-	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerId, request)
+	containerInfo, err := fm.Cadvisor().Client().DockerContainer(containerID, request)
 	require.NoError(t, err)
-	sanityCheck(containerId, containerInfo, t)
+	sanityCheck(containerID, containerInfo, t)
+
+	stat := containerInfo.Stats[0]
+	ifaceStats := stat.Network.InterfaceStats
+	// macOS we have more than one interface, since traffic is
+	// only on eth0 we need to pick that one
+	if len(stat.Network.Interfaces) > 0 {
+		for _, iface := range stat.Network.Interfaces {
+			if iface.Name == "eth0" {
+				ifaceStats = iface
+			}
+		}
+	}
 
 	// Checks for NetworkStats.
-	stat := containerInfo.Stats[0]
 	assert := assert.New(t)
-	assert.NotEqual(0, stat.Network.TxBytes, "Network tx bytes should not be zero")
-	assert.NotEqual(0, stat.Network.TxPackets, "Network tx packets should not be zero")
-	assert.NotEqual(0, stat.Network.RxBytes, "Network rx bytes should not be zero")
-	assert.NotEqual(0, stat.Network.RxPackets, "Network rx packets should not be zero")
-	assert.NotEqual(stat.Network.RxBytes, stat.Network.TxBytes, "Network tx and rx bytes should not be equal")
-	assert.NotEqual(stat.Network.RxPackets, stat.Network.TxPackets, "Network tx and rx packets should not be equal")
+	assert.NotEqual(0, ifaceStats.TxBytes, "Network tx bytes should not be zero")
+	assert.NotEqual(0, ifaceStats.TxPackets, "Network tx packets should not be zero")
+	assert.NotEqual(0, ifaceStats.RxBytes, "Network rx bytes should not be zero")
+	assert.NotEqual(0, ifaceStats.RxPackets, "Network rx packets should not be zero")
+	assert.NotEqual(ifaceStats.RxBytes, ifaceStats.TxBytes, fmt.Sprintf("Network tx (%d) and rx (%d) bytes should not be equal", ifaceStats.TxBytes, ifaceStats.RxBytes))
+	assert.NotEqual(ifaceStats.RxPackets, ifaceStats.TxPackets, fmt.Sprintf("Network tx (%d) and rx (%d) packets should not be equal", ifaceStats.TxPackets, ifaceStats.RxPackets))
 }
 
 func TestDockerFilesystemStats(t *testing.T) {
@@ -325,8 +341,8 @@ func TestDockerFilesystemStats(t *testing.T) {
 	if fm.Hostname().Host != "localhost" {
 		dockerCmd = fmt.Sprintf("'%s'", dockerCmd)
 	}
-	containerId := fm.Docker().RunBusybox("/bin/sh", "-c", dockerCmd)
-	waitForContainer(containerId, fm)
+	containerID := fm.Docker().RunBusybox("/bin/sh", "-c", dockerCmd)
+	waitForContainer(containerID, fm)
 	request := &v2.RequestOptions{
 		IdType: v2.TypeDocker,
 		Count:  1,
@@ -339,7 +355,7 @@ func TestDockerFilesystemStats(t *testing.T) {
 	pass := false
 	// We need to wait for the `dd` operation to complete.
 	for i := 0; i < 10; i++ {
-		containerInfo, err := fm.Cadvisor().ClientV2().Stats(containerId, request)
+		containerInfo, err := fm.Cadvisor().ClientV2().Stats(containerID, request)
 		if err != nil {
 			t.Logf("%v stats unavailable - %v", time.Now().String(), err)
 			t.Logf("retrying after %s...", sleepDuration.String())
@@ -354,7 +370,7 @@ func TestDockerFilesystemStats(t *testing.T) {
 		for _, cInfo := range containerInfo {
 			info = cInfo
 		}
-		sanityCheckV2(containerId, info, t)
+		sanityCheckV2(containerID, info, t)
 
 		require.NotNil(t, info.Stats[0], "got info: %+v", info)
 		require.NotNil(t, info.Stats[0].Filesystem, "got info: %+v", info)
